@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -28,12 +28,8 @@
  * This file contains file-related functions.
  *
  * @author Michael Jennings <mej@eterm.org>
- * $Revision: 1.11 $
- * $Date: 2004/07/23 21:38:39 $
  */
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: file.c,v 1.11 2004/07/23 21:38:39 mej Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -67,20 +63,20 @@ spiftool_temp_file(spif_charptr_t ftemplate, size_t len)
     int fd;
     mode_t m;
 
-    ASSERT_RVAL(!SPIF_PTR_ISNULL(ftemplate), SPIF_CAST_C(int) -1);
-    ASSERT_RVAL(len > 0, SPIF_CAST_C(int) -1);
+    ASSERT_RVAL(!SPIF_PTR_ISNULL(ftemplate), (int) -1);
+    ASSERT_RVAL(len > 0, (int) -1);
     if (getenv("TMPDIR")) {
-        snprintf(SPIF_CAST_C(char *) buff, sizeof(buff),
+        snprintf((char *) buff, sizeof(buff),
                  "%s/%sXXXXXX", getenv("TMPDIR"), ftemplate);
     } else if (getenv("TMP")) {
-        snprintf(SPIF_CAST_C(char *) buff, sizeof(buff),
+        snprintf((char *) buff, sizeof(buff),
                  "%s/%sXXXXXX", getenv("TMP"), ftemplate);
     } else {
-        snprintf(SPIF_CAST_C(char *) buff, sizeof(buff),
+        snprintf((char *) buff, sizeof(buff),
                  "/tmp/%sXXXXXX", ftemplate);
     }
     m = umask(0077);
-    fd = mkstemp(SPIF_CAST_C(char *) buff);
+    fd = mkstemp((char *) buff);
     umask(m);
     if ((fd < 0) || fchmod(fd, (S_IRUSR | S_IWUSR))) {
         return (-1);
