@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,8 +21,6 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: mbuff.c,v 1.1 2004/12/15 00:00:22 mej Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -66,14 +64,17 @@ static SPIF_CONST_TYPE(mbuffclass) mb_class = {
     (spif_func_t) spif_mbuff_rindex,
     (spif_func_t) spif_mbuff_splice,
     (spif_func_t) spif_mbuff_splice_from_ptr,
+    (spif_func_t) spif_mbuff_sprintf,
     (spif_func_t) spif_mbuff_subbuff,
     (spif_func_t) spif_mbuff_subbuff_to_ptr,
     (spif_func_t) spif_mbuff_trim
 };
-SPIF_TYPE(class) SPIF_CLASS_VAR(mbuff) = SPIF_CAST(class) &mb_class;
+SPIF_TYPE(class) SPIF_CLASS_VAR(mbuff) = (spif_class_t) &mb_class;
 SPIF_TYPE(mbuffclass) SPIF_MBUFFCLASS_VAR(mbuff) = &mb_class;
 /* *INDENT-ON* */
 
+static const size_t buff_inc = 4096;
+
 spif_mbuff_t
 spif_mbuff_new(void)
 {
@@ -82,7 +83,7 @@ spif_mbuff_new(void)
     self = SPIF_ALLOC(mbuff);
     if (!spif_mbuff_init(self)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(mbuff);
+        self = (spif_mbuff_t) NULL;
     }
     return self;
 }
@@ -95,7 +96,7 @@ spif_mbuff_new_from_ptr(spif_byteptr_t old, spif_memidx_t len)
     self = SPIF_ALLOC(mbuff);
     if (!spif_mbuff_init_from_ptr(self, old, len)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(mbuff);
+        self = (spif_mbuff_t) NULL;
     }
     return self;
 }
@@ -108,7 +109,7 @@ spif_mbuff_new_from_buff(spif_byteptr_t buff, spif_memidx_t len, spif_memidx_t s
     self = SPIF_ALLOC(mbuff);
     if (!spif_mbuff_init_from_buff(self, buff, len, size)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(mbuff);
+        self = (spif_mbuff_t) NULL;
     }
     return self;
 }
@@ -121,7 +122,7 @@ spif_mbuff_new_from_fp(FILE * fp)
     self = SPIF_ALLOC(mbuff);
     if (!spif_mbuff_init_from_fp(self, fp)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(mbuff);
+        self = (spif_mbuff_t) NULL;
     }
     return self;
 }
@@ -134,7 +135,7 @@ spif_mbuff_new_from_fd(int fd)
     self = SPIF_ALLOC(mbuff);
     if (!spif_mbuff_init_from_fd(self, fd)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(mbuff);
+        self = (spif_mbuff_t) NULL;
     }
     return self;
 }
@@ -145,7 +146,7 @@ spif_mbuff_init(spif_mbuff_t self)
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_MBUFFCLASS_VAR(mbuff)));
-    self->buff = SPIF_NULL_TYPE(byteptr);
+    self->buff = (spif_byteptr_t) NULL;
     self->len = 0;
     self->size = 0;
     return TRUE;
@@ -155,11 +156,11 @@ spif_bool_t
 spif_mbuff_init_from_ptr(spif_mbuff_t self, spif_byteptr_t old, spif_memidx_t len)
 {
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
-    REQUIRE_RVAL((old != SPIF_NULL_TYPE(byteptr)), spif_mbuff_init(self));
+    REQUIRE_RVAL((old != (spif_byteptr_t) NULL), spif_mbuff_init(self));
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_MBUFFCLASS_VAR(mbuff)));
     self->len = self->size = len;
-    self->buff = SPIF_CAST(byteptr) MALLOC(self->size);
+    self->buff = (spif_byteptr_t) MALLOC(self->size);
     memcpy(self->buff, old, self->len);
     return TRUE;
 }
@@ -170,14 +171,14 @@ spif_mbuff_init_from_buff(spif_mbuff_t self, spif_byteptr_t buff, spif_memidx_t
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_MBUFFCLASS_VAR(mbuff)));
-    if (buff != SPIF_NULL_TYPE(byteptr)) {
+    if (buff != (spif_byteptr_t) NULL) {
         self->len = len;
     } else {
         self->len = 0;
     }
     self->size = MAX(size, self->len);
-    self->buff = SPIF_CAST(byteptr) MALLOC(self->size);
-    if (buff != SPIF_NULL_TYPE(byteptr)) {
+    self->buff = (spif_byteptr_t) MALLOC(self->size);
+    if (buff != (spif_byteptr_t) NULL) {
         memcpy(self->buff, buff, self->len);
     }
     return TRUE;
@@ -186,30 +187,58 @@ spif_mbuff_init_from_buff(spif_mbuff_t self, spif_byteptr_t buff, spif_memidx_t
 spif_bool_t
 spif_mbuff_init_from_fp(spif_mbuff_t self, FILE *fp)
 {
-    spif_byteptr_t p;
     long file_pos;
     spif_memidx_t file_size;
 
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
-    ASSERT_RVAL((fp != SPIF_NULL_TYPE_C(FILE *)), FALSE);
+    ASSERT_RVAL((fp != (FILE *) NULL), FALSE);
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_MBUFFCLASS_VAR(mbuff)));
 
     file_pos = ftell(fp);
-    fseek(fp, 0L, SEEK_END);
-    file_size = ftell(fp);
-    fseek(fp, file_pos, SEEK_SET);
-    LOWER_BOUND(file_size, 0);
-    if (file_size == 0) {
-        spif_mbuff_init(self);
-        return FALSE;
-    }
-    self->len = self->size = file_size;
-    self->buff = SPIF_CAST(byteptr) MALLOC(self->size);
+    LOWER_BOUND(file_pos, 0);
+    if (fseek(fp, 0L, SEEK_END) < 0) {
+        spif_byteptr_t p;
+        size_t cnt = 0;
 
-    if (fread(p, file_size, 1, fp) < 1) {
-        FREE(self->buff);
-        return FALSE;
+        D_OBJ(("Unable to seek to EOF -- %s.\n", strerror(errno)));
+        self->size = buff_inc;
+        self->len = 0;
+        self->buff = (spif_byteptr_t) MALLOC(self->size);
+
+        for (p = self->buff; (cnt = fread(p, 1, buff_inc, fp)) > 0; p += buff_inc) {
+            self->len += cnt;
+            if (feof(fp)) {
+                break;
+            } else if (ferror(fp)) {
+                libast_print_warning("read failed:  %s.\n", strerror(errno));
+                break;
+            } else {
+                self->size += buff_inc;
+                self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
+            }
+        }
+        self->size = self->len;
+        if (self->size) {
+            self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
+        } else {
+            FREE(self->buff);
+        }
+    } else {
+        file_size = ftell(fp);
+        fseek(fp, file_pos, SEEK_SET);
+        LOWER_BOUND(file_size, 0);
+        if (file_size <= 0) {
+            spif_mbuff_init(self);
+            return FALSE;
+        }
+        self->len = self->size = file_size;
+        self->buff = (spif_byteptr_t) MALLOC(self->size);
+
+        if (fread(self->buff, file_size, 1, fp) < 1) {
+            FREE(self->buff);
+            return FALSE;
+        }
     }
     return TRUE;
 }
@@ -226,20 +255,41 @@ spif_mbuff_init_from_fd(spif_mbuff_t self, int fd)
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_MBUFFCLASS_VAR(mbuff)));
 
-    file_pos = lseek(fd, SPIF_CAST_C(off_t) 0, SEEK_CUR);
-    file_size = SPIF_CAST(memidx) lseek(fd, SPIF_CAST_C(off_t) 0, SEEK_END);
+    file_pos = lseek(fd, (off_t) 0, SEEK_CUR);
+    file_size = (spif_memidx_t) lseek(fd, (off_t) 0, SEEK_END);
     lseek(fd, file_pos, SEEK_SET);
-    LOWER_BOUND(file_size, 0);
-    if (file_size == 0) {
-        spif_mbuff_init(self);
-        return FALSE;
-    }
-    self->len = self->size = file_size;
-    self->buff = SPIF_CAST(byteptr) MALLOC(self->size);
+    if (file_size < 0) {
+        spif_byteptr_t p;
+        size_t cnt = 0;
 
-    if (read(fd, p, file_size) < 1) {
-        FREE(self->buff);
-        return FALSE;
+        D_OBJ(("Unable to seek to EOF -- %s.\n", strerror(errno)));
+        self->size = buff_inc;
+        self->len = 0;
+        self->buff = (spif_byteptr_t) MALLOC(self->size);
+
+        for (p = self->buff; (cnt = read(fd, p, buff_inc)) > 0; p += buff_inc) {
+            self->len += cnt;
+            if (cnt < buff_inc) {
+                break;
+            } else {
+                self->size += buff_inc;
+                self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
+            }
+        }
+        self->size = self->len;
+        if (self->size) {
+            self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
+        } else {
+            FREE(self->buff);
+        }
+    } else {
+        self->len = self->size = file_size;
+        self->buff = (spif_byteptr_t) MALLOC(self->size);
+
+        if (read(fd, p, file_size) < 1) {
+            FREE(self->buff);
+            return FALSE;
+        }
     }
     return TRUE;
 }
@@ -252,7 +302,7 @@ spif_mbuff_done(spif_mbuff_t self)
         FREE(self->buff);
         self->len = 0;
         self->size = 0;
-        self->buff = SPIF_NULL_TYPE(byteptr);
+        self->buff = (spif_byteptr_t) NULL;
     }
     return TRUE;
 }
@@ -278,9 +328,9 @@ spif_mbuff_show(spif_mbuff_t self, spif_byteptr_t name, spif_str_t buff, size_t
     }
 
     memset(tmp, ' ', indent);
-    snprintf(SPIF_CHARPTR_C(tmp) + indent, sizeof(tmp) - indent,
+    snprintf((char *) tmp + indent, sizeof(tmp) - indent,
              "(spif_mbuff_t) %s:  %10p (length %lu, size %lu) {\n",
-             name, SPIF_CAST(ptr) self, SPIF_CAST(ulong) self->len, SPIF_CAST(ulong) self->size);
+             name, (spif_ptr_t) self, (spif_ulong_t) self->len, (spif_ulong_t) self->size);
     if (SPIF_MBUFF_ISNULL(buff)) {
         buff = spif_str_new_from_ptr(tmp);
     } else {
@@ -291,22 +341,28 @@ spif_mbuff_show(spif_mbuff_t self, spif_byteptr_t name, spif_str_t buff, size_t
     for (j = 0; j < self->len; j += 8) {
         spif_memidx_t k, l, len;
 
-        snprintf(SPIF_CHARPTR_C(tmp) + indent + 2, sizeof(tmp) - indent - 2, "%0lx    ", SPIF_CAST(long) j);
-        len = strlen(SPIF_CHARPTR_C(tmp));
-        if (len + ((8 * 3) + 3 + 8 + 1) < sizeof(tmp)) {
+        snprintf((char *) tmp + indent + 2, sizeof(tmp) - indent - 2, "0x%08lx    ", (spif_long_t) j);
+        len = strlen((char *) tmp);
+        if ((sizeof(tmp) - indent - 2) > 46) {
+            spif_char_t buffr[9];
+
             l = ((self->len - j < 8) ? (self->len - j) : (8));
+            memcpy(buffr, self->buff + j, l);
+            memset(buffr + l, 0, 9 - l);
             for (k = 0; k < l; k++) {
-                sprintf(SPIF_CHARPTR_C(tmp) + len, "%02x ", self->buff[j + k]);
+                sprintf((char *) tmp + 14 + (k * 3), "%02x ", self->buff[j + k]);
             }
             for (; k < 8; k++) {
-                strcat(SPIF_CHARPTR_C(tmp) + len, "   ");
+                strcat((char *) tmp + 14, "   ");
             }
-            sprintf(SPIF_CHARPTR_C(tmp) + len, "%-8s\n", spiftool_safe_str(SPIF_CAST(charptr) (self->buff + j), l));
+            sprintf((char *) tmp + 38, "%-8s\n", spiftool_safe_str((spif_charptr_t) (buffr), l));
+        } else {
+            snprintf((char *) tmp + indent + 2, sizeof(tmp) - indent - 2, "!X!");
         }
         spif_str_append_from_ptr(buff, tmp);
     }
 
-    snprintf(SPIF_CHARPTR_C(tmp) + indent, sizeof(tmp) - indent, "}\n");
+    snprintf((char *) tmp + indent, sizeof(tmp) - indent, "}\n");
     spif_str_append_from_ptr(buff, tmp);
     return buff;
 }
@@ -322,10 +378,10 @@ spif_mbuff_dup(spif_mbuff_t self)
 {
     spif_mbuff_t tmp;
 
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), SPIF_NULL_TYPE(mbuff));
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (spif_mbuff_t) NULL);
     tmp = SPIF_ALLOC(mbuff);
     memcpy(tmp, self, SPIF_SIZEOF_TYPE(mbuff));
-    tmp->buff = SPIF_CAST(byteptr) MALLOC(self->size);
+    tmp->buff = (spif_byteptr_t) MALLOC(self->size);
     memcpy(tmp->buff, self->buff, self->size);
     tmp->len = self->len;
     tmp->size = self->size;
@@ -335,7 +391,7 @@ spif_mbuff_dup(spif_mbuff_t self)
 spif_classname_t
 spif_mbuff_type(spif_mbuff_t self)
 {
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), SPIF_CAST(classname) SPIF_NULLSTR_TYPE(classname));
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (spif_classname_t) SPIF_NULLSTR_TYPE(classname));
     return SPIF_OBJ_CLASSNAME(self);
 }
 
@@ -346,7 +402,7 @@ spif_mbuff_append(spif_mbuff_t self, spif_mbuff_t other)
     REQUIRE_RVAL(!SPIF_MBUFF_ISNULL(other), FALSE);
     if (other->size && other->len) {
         self->size += other->size;
-        self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, self->size);
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
         memcpy(self->buff + self->len, SPIF_MBUFF_BUFF(other), other->len);
         self->len += other->len;
     }
@@ -357,10 +413,10 @@ spif_bool_t
 spif_mbuff_append_from_ptr(spif_mbuff_t self, spif_byteptr_t other, spif_memidx_t len)
 {
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
-    REQUIRE_RVAL((other != SPIF_NULL_TYPE(byteptr)), FALSE);
+    REQUIRE_RVAL((other != (spif_byteptr_t) NULL), FALSE);
     if (len) {
         self->size += len;
-        self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, self->size);
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
         memcpy(self->buff + self->len, other, len);
         self->len += len;
     }
@@ -400,13 +456,13 @@ spif_mbuff_find(spif_mbuff_t self, spif_mbuff_t other)
 {
     spif_byteptr_t tmp;
 
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (SPIF_CAST(memidx) -1));
-    REQUIRE_RVAL(!SPIF_MBUFF_ISNULL(other), (SPIF_CAST(memidx) -1));
-    tmp = SPIF_CAST(byteptr) memmem(SPIF_MBUFF_BUFF(self), self->len, SPIF_MBUFF_BUFF(other), other->len);
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), ((spif_memidx_t) -1));
+    REQUIRE_RVAL(!SPIF_MBUFF_ISNULL(other), ((spif_memidx_t) -1));
+    tmp = (spif_byteptr_t) memmem(SPIF_MBUFF_BUFF(self), self->len, SPIF_MBUFF_BUFF(other), other->len);
     if (tmp) {
-        return SPIF_CAST(memidx) (SPIF_CAST(long) tmp - SPIF_CAST(long) (SPIF_MBUFF_BUFF(self)));
+        return (spif_memidx_t) ((spif_long_t) tmp - (spif_long_t) (SPIF_MBUFF_BUFF(self)));
     } else {
-        return SPIF_CAST(memidx) (self->len);
+        return (spif_memidx_t) (self->len);
     }
 }
 
@@ -415,13 +471,13 @@ spif_mbuff_find_from_ptr(spif_mbuff_t self, spif_byteptr_t other, spif_memidx_t
 {
     spif_byteptr_t tmp;
 
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (SPIF_CAST(memidx) -1));
-    REQUIRE_RVAL((other != SPIF_NULL_TYPE(byteptr)), (SPIF_CAST(memidx) -1));
-    tmp = SPIF_CAST(byteptr) memmem(SPIF_MBUFF_BUFF(self), self->len, other, len);
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), ((spif_memidx_t) -1));
+    REQUIRE_RVAL((other != (spif_byteptr_t) NULL), ((spif_memidx_t) -1));
+    tmp = (spif_byteptr_t) memmem(SPIF_MBUFF_BUFF(self), self->len, other, len);
     if (tmp) {
-        return SPIF_CAST(memidx) (SPIF_CAST(long) tmp - SPIF_CAST(long) (SPIF_MBUFF_BUFF(self)));
+        return (spif_memidx_t) ((spif_long_t) tmp - (spif_long_t) (SPIF_MBUFF_BUFF(self)));
     } else {
-        return SPIF_CAST(memidx) (self->len);
+        return (spif_memidx_t) (self->len);
     }
 }
 
@@ -431,9 +487,9 @@ spif_mbuff_index(spif_mbuff_t self, spif_uint8_t c)
     spif_byteptr_t tmp;
     spif_memidx_t i;
 
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (SPIF_CAST(memidx) -1));
-    for (tmp = self->buff, i = 0; (*tmp & (~c)) && (i < self->len); i++, tmp++);
-    return SPIF_CAST(memidx) (SPIF_CAST(long) tmp - SPIF_CAST(long) self->buff);
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), ((spif_memidx_t) -1));
+    for (tmp = self->buff, i = 0; ((int) *tmp != (int) (c)) && (i < self->len); i++, tmp++);
+    return (spif_memidx_t) ((spif_long_t) tmp - (spif_long_t) self->buff);
 }
 
 spif_cmp_t
@@ -462,7 +518,7 @@ spif_mbuff_prepend(spif_mbuff_t self, spif_mbuff_t other)
     REQUIRE_RVAL(!SPIF_MBUFF_ISNULL(other), FALSE);
     if (other->size && other->len) {
         self->size += other->size;
-        self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, self->size);
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
         memmove(self->buff + other->len, self->buff, self->len);
         memcpy(self->buff, SPIF_MBUFF_BUFF(other), other->len);
         self->len += other->len;
@@ -474,10 +530,10 @@ spif_bool_t
 spif_mbuff_prepend_from_ptr(spif_mbuff_t self, spif_byteptr_t other, spif_memidx_t len)
 {
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
-    REQUIRE_RVAL((other != SPIF_NULL_TYPE(byteptr)), FALSE);
+    REQUIRE_RVAL((other != (spif_byteptr_t) NULL), FALSE);
     if (len) {
         self->size += len;
-        self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, self->size);
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
         memmove(self->buff + len, self->buff, self->len);
         memcpy(self->buff, other, len);
         self->len += len;
@@ -492,7 +548,7 @@ spif_mbuff_reverse(spif_mbuff_t self)
     int i, j;
 
     ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
-    REQUIRE_RVAL(self->buff != SPIF_NULL_TYPE(byteptr), FALSE);
+    REQUIRE_RVAL(self->buff != (spif_byteptr_t) NULL, FALSE);
 
     for (j = 0, i = self->len - 1; i > j; i--, j++) {
         SWAP(tmp[j], tmp[i]);
@@ -504,15 +560,14 @@ spif_memidx_t
 spif_mbuff_rindex(spif_mbuff_t self, spif_uint8_t c)
 {
     spif_byteptr_t tmp;
-    spif_memidx_t i;
 
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (SPIF_CAST(memidx) -1));
-    for (tmp = self->buff + self->len, i = 0; (*tmp & (~c)) && (i < self->len); i++, tmp--);
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), ((spif_memidx_t) -1));
+    for (tmp = self->buff + self->len - 1; (*tmp != c) && (tmp >= self->buff); tmp--);
 
     if ((tmp == self->buff) && (*tmp != c)) {
-        return SPIF_CAST(memidx) (self->len);
+        return (spif_memidx_t) (self->len);
     } else {
-        return SPIF_CAST(memidx) (SPIF_CAST(long) tmp - SPIF_CAST(long) self->buff);
+        return (spif_memidx_t) ((spif_long_t) tmp - (spif_long_t) self->buff);
     }
 }
 
@@ -534,8 +589,8 @@ spif_mbuff_splice(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t cnt, spif_
     REQUIRE_RVAL(cnt >= 0, FALSE);
     REQUIRE_RVAL(cnt <= (self->len - idx), FALSE);
 
-    newsize = self->len + ((SPIF_MBUFF_ISNULL(other)) ? (0) : (other->len)) - cnt + 1;
-    ptmp = tmp = SPIF_CAST(byteptr) MALLOC(newsize);
+    newsize = self->len + ((SPIF_MBUFF_ISNULL(other)) ? (0) : (other->len)) - cnt;
+    ptmp = tmp = (spif_byteptr_t) MALLOC(newsize);
     if (idx > 0) {
         memcpy(tmp, self->buff, idx);
         ptmp += idx;
@@ -546,7 +601,7 @@ spif_mbuff_splice(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t cnt, spif_
     }
     memcpy(ptmp, self->buff + idx + cnt, self->len - idx - cnt + 1);
     if (self->size < newsize) {
-        self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, newsize);
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, newsize);
         self->size = newsize;
     }
     self->len = newsize;
@@ -577,7 +632,7 @@ spif_mbuff_splice_from_ptr(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t c
     REQUIRE_RVAL(cnt <= (self->len - idx), FALSE);
 
     newsize = self->len + len - cnt;
-    ptmp = tmp = SPIF_CAST(byteptr) MALLOC(newsize);
+    ptmp = tmp = (spif_byteptr_t) MALLOC(newsize);
     if (idx > 0) {
         memcpy(tmp, self->buff, idx);
         ptmp += idx;
@@ -588,7 +643,7 @@ spif_mbuff_splice_from_ptr(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t c
     }
     memcpy(ptmp, self->buff + idx + cnt, self->len - idx - cnt);
     if (self->size < newsize) {
-        self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, newsize);
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, newsize);
         self->size = newsize;
     }
     self->len = newsize;
@@ -597,19 +652,60 @@ spif_mbuff_splice_from_ptr(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t c
     return TRUE;
 }
 
+spif_bool_t
+spif_mbuff_sprintf(spif_mbuff_t self, spif_charptr_t format, ...)
+{
+    va_list ap;
+
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), FALSE);
+    va_start(ap, format);
+    if (self->buff != (spif_byteptr_t) NULL) {
+        spif_mbuff_done(self);
+    }
+    if (!format) {
+        return FALSE;
+    } else if (*format == 0) {
+        return TRUE;
+    } else {
+        int c;
+        char buff[2];
+
+        va_start(ap, format);
+        c = vsnprintf(buff, sizeof(buff), format, ap);
+        va_end(ap);
+        if (c <= 0) {
+            return FALSE;
+        } else {
+            c++;
+            self->size = c;
+            self->buff = (spif_charptr_t) MALLOC(self->size);
+            va_start(ap, format);
+            c = vsnprintf(self->buff, self->size, format, ap);
+            va_end(ap);
+            if (c > -1 && c < self->size) {
+                self->len = c;
+                return TRUE;
+            } else {
+                return FALSE;
+            }
+        }
+    }
+    ASSERT_NOTREACHED_RVAL(FALSE);
+}
+
 spif_mbuff_t
 spif_mbuff_subbuff(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t cnt)
 {
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), SPIF_NULL_TYPE(mbuff));
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (spif_mbuff_t) NULL);
     if (idx < 0) {
         idx = self->len + idx;
     }
-    REQUIRE_RVAL(idx >= 0, SPIF_NULL_TYPE(mbuff));
-    REQUIRE_RVAL(idx < self->len, SPIF_NULL_TYPE(mbuff));
+    REQUIRE_RVAL(idx >= 0, (spif_mbuff_t) NULL);
+    REQUIRE_RVAL(idx < self->len, (spif_mbuff_t) NULL);
     if (cnt <= 0) {
         cnt = self->len - idx + cnt;
     }
-    REQUIRE_RVAL(cnt >= 0, SPIF_NULL_TYPE(mbuff));
+    REQUIRE_RVAL(cnt >= 0, (spif_mbuff_t) NULL);
     UPPER_BOUND(cnt, self->len - idx);
     return spif_mbuff_new_from_buff(self->buff + idx, cnt, cnt);
 }
@@ -619,19 +715,19 @@ spif_mbuff_subbuff_to_ptr(spif_mbuff_t self, spif_memidx_t idx, spif_memidx_t cn
 {
     spif_byteptr_t newmbuff;
 
-    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), SPIF_NULL_TYPE(byteptr));
+    ASSERT_RVAL(!SPIF_MBUFF_ISNULL(self), (spif_byteptr_t) NULL);
     if (idx < 0) {
         idx = self->len + idx;
     }
-    REQUIRE_RVAL(idx >= 0, SPIF_NULL_TYPE(byteptr));
-    REQUIRE_RVAL(idx < self->len, SPIF_NULL_TYPE(byteptr));
+    REQUIRE_RVAL(idx >= 0, (spif_byteptr_t) NULL);
+    REQUIRE_RVAL(idx < self->len, (spif_byteptr_t) NULL);
     if (cnt <= 0) {
         cnt = self->len - idx + cnt;
     }
-    REQUIRE_RVAL(cnt >= 0, SPIF_NULL_TYPE(byteptr));
+    REQUIRE_RVAL(cnt >= 0, (spif_byteptr_t) NULL);
     UPPER_BOUND(cnt, self->len - idx);
 
-    newmbuff = SPIF_CAST(byteptr) MALLOC(cnt + 1);
+    newmbuff = (spif_byteptr_t) MALLOC(cnt + 1);
     memcpy(newmbuff, SPIF_MBUFF_BUFF(self) + idx, cnt);
     newmbuff[cnt] = 0;
     return newmbuff;
@@ -650,11 +746,14 @@ spif_mbuff_trim(spif_mbuff_t self)
     if (start > end) {
         return spif_mbuff_done(self);
     }
-    *(++end) = 0;
-    self->len = (spif_memidx_t) (end - start);
-    self->size = self->len + 1;
-    memmove(self->buff, start, self->size);
-    self->buff = SPIF_CAST(byteptr) REALLOC(self->buff, self->size);
+    self->len = (spif_memidx_t) (end - start + 1);
+    if (start > self->buff) {
+        memmove(self->buff, start, self->len);
+    }
+    if (self->size != self->len) {
+        self->size = self->len;
+        self->buff = (spif_byteptr_t) REALLOC(self->buff, self->size);
+    }
     return TRUE;
 }
 
