@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -28,12 +28,8 @@
  * This file contains the basic object class.
  *
  * @author Michael Jennings <mej@eterm.org>
- * $Revision: 1.27 $
- * $Date: 2004/07/23 21:38:39 $
  */
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: obj.c,v 1.27 2004/07/23 21:38:39 mej Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -121,7 +117,7 @@ spif_obj_new(void)
     self = SPIF_ALLOC(obj);
     if (!spif_obj_init(self)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(obj);
+        self = (spif_obj_t) NULL;
     }
     return self;
 }
@@ -249,9 +245,9 @@ spif_obj_show(spif_obj_t self, spif_charptr_t name, spif_str_t buff, size_t inde
     }
 
     memset(tmp, ' ', indent);
-    snprintf(SPIF_CAST_C(char *) tmp + indent, sizeof(tmp) - indent,
+    snprintf((char *) tmp + indent, sizeof(tmp) - indent,
              "(spif_obj_t) %s:  %10p \"%s\"\n",
-             name, SPIF_CAST(ptr) self, SPIF_OBJ_CLASSNAME(self));
+             name, (spif_ptr_t) self, SPIF_OBJ_CLASSNAME(self));
     if (SPIF_STR_ISNULL(buff)) {
         buff = spif_str_new_from_ptr(tmp);
     } else {
@@ -283,7 +279,7 @@ spif_cmp_t
 spif_obj_comp(spif_obj_t self, spif_obj_t other)
 {
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    return SPIF_CMP_FROM_INT(SPIF_CAST(ulong) self - SPIF_CAST(ulong) other);
+    return SPIF_CMP_FROM_INT((spif_ulong_t) self - (spif_ulong_t) other);
 }
 
 /**
@@ -311,7 +307,7 @@ spif_obj_dup(spif_obj_t self)
 {
     spif_obj_t tmp;
 
-    ASSERT_RVAL(!SPIF_OBJ_ISNULL(self), SPIF_NULL_TYPE(obj));
+    ASSERT_RVAL(!SPIF_OBJ_ISNULL(self), (spif_obj_t) NULL);
     tmp = spif_obj_new();
     memcpy(tmp, self, SPIF_SIZEOF_TYPE(obj));
     return tmp;
@@ -334,7 +330,7 @@ spif_obj_dup(spif_obj_t self)
 spif_classname_t
 spif_obj_type(spif_obj_t self)
 {
-    ASSERT_RVAL(!SPIF_OBJ_ISNULL(self), SPIF_NULL_TYPE(classname));
+    ASSERT_RVAL(!SPIF_OBJ_ISNULL(self), (spif_classname_t) NULL);
     return SPIF_OBJ_CLASSNAME(self);
 }
 
@@ -361,7 +357,7 @@ spif_obj_type(spif_obj_t self)
 spif_class_t
 spif_obj_get_class(spif_obj_t self)
 {
-    ASSERT_RVAL(!SPIF_OBJ_ISNULL(self), SPIF_NULL_TYPE(class));
+    ASSERT_RVAL(!SPIF_OBJ_ISNULL(self), (spif_class_t) NULL);
     return SPIF_OBJ_CLASS(self);
 }
 
