@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -29,12 +29,8 @@
  * option parser.
  *
  * @author Michael Jennings <mej@eterm.org>
- * $Revision: 1.18 $
- * $Date: 2004/10/26 18:01:53 $
  */
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: options.c,v 1.18 2004/10/26 18:01:53 mej Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -160,7 +156,7 @@ spifopt_usage(void)
             printf("      ");
         }
         printf("--%s", SPIFOPT_OPT_LONG(i));
-        for (col = strlen(SPIF_CAST_C(char *) SPIFOPT_OPT_LONG(i)); col < l_long; col++) {
+        for (col = strlen((char *) SPIFOPT_OPT_LONG(i)); col < l_long; col++) {
             printf(" ");
         }
         printf("  %-6s %s\n", get_option_type_string(SPIFOPT_OPT_TYPE(i)), SPIFOPT_OPT_DESC(i));
@@ -190,9 +186,9 @@ find_long_option(spif_charptr_t opt)
     for (j = 0; j < SPIFOPT_NUMOPTS_GET(); j++) {
         size_t l;
 
-        l = strlen(SPIF_CHARPTR_C(SPIFOPT_OPT_LONG(j)));
+        l = strlen((char *) SPIFOPT_OPT_LONG(j));
         /* Look for matches to the part before the =, if any. */
-        if (!strncasecmp(SPIF_CHARPTR_C(SPIFOPT_OPT_LONG(j)), SPIF_CHARPTR_C(opt), l)
+        if (!strncasecmp((char *) SPIFOPT_OPT_LONG(j), (char *) opt, l)
             && (opt[l] == '=' || !opt[l])) {
             /* Got one. */
             D_OPTIONS(("Match found at %d:  %s == %s\n", j, SPIFOPT_OPT_LONG(j), opt));
@@ -257,7 +253,7 @@ find_value_long(spif_charptr_t arg, spif_charptr_t next_arg, spif_charptr_t hase
 {
     spif_charptr_t val_ptr;
 
-    if ((val_ptr = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(arg), '='))) != NULL) {
+    if ((val_ptr = SPIF_CHARPTR(strchr((char *)arg, '=')))) {
         val_ptr++;
         *hasequal = 1;
     } else {
@@ -291,7 +287,7 @@ find_value_short(spif_charptr_t arg, spif_charptr_t next_arg)
 
     if (arg[1]) {
         val_ptr = arg + 1;
-    } else if (next_arg != NULL) {
+    } else if (next_arg) {
         val_ptr = next_arg;
     }
     D_OPTIONS(("val_ptr == %10.8p \"%s\"\n", val_ptr, NONULL(val_ptr)));
@@ -427,7 +423,7 @@ static void
 handle_integer(spif_int32_t n, spif_charptr_t val_ptr)
 {
     D_OPTIONS(("Integer option detected\n"));
-    *((int *) SPIFOPT_OPT_VALUE(n)) = strtol(SPIF_CHARPTR_C(val_ptr), (char **) NULL, 0);
+    *((int *) SPIFOPT_OPT_VALUE(n)) = strtol((char *) val_ptr, (char **) NULL, 0);
 }
 
 /**
@@ -448,7 +444,7 @@ static void
 handle_string(spif_int32_t n, spif_charptr_t val_ptr)
 {
     D_OPTIONS(("String option detected\n"));
-    *((const char **) SPIFOPT_OPT_VALUE(n)) = SPIF_CAST_C(char *) STRDUP(val_ptr);
+    *((const char **) SPIFOPT_OPT_VALUE(n)) = (char *) STRDUP(val_ptr);
 }
 
 
@@ -480,28 +476,30 @@ handle_arglist(spif_int32_t n, spif_charptr_t val_ptr, unsigned char hasequal,
     D_OPTIONS(("Argument list option detected\n"));
     if (hasequal) {
         /* There's an equals sign, so just parse the rest of this option into words. */
-        tmp = SPIF_CAST_PTR(charptr) MALLOC(sizeof(spif_charptr_t) * (spiftool_num_words(val_ptr) + 1));
+        tmp = (spif_charptr_t *) MALLOC(sizeof(spif_charptr_t) * (spiftool_num_words(val_ptr) + 1));
 
         for (k = 0; val_ptr; k++) {
             tmp[k] = spiftool_get_word(1, val_ptr);
             val_ptr = spiftool_get_pword(2, val_ptr);
             D_OPTIONS(("tmp[%d] == %s\n", k, tmp[k]));
         }
-        tmp[k] = SPIF_NULL_TYPE(charptr);
-        *(SPIF_CAST_C(spif_charptr_t **) SPIFOPT_OPT_VALUE(n)) = tmp;
+        tmp[k] = (spif_charptr_t) NULL;
+        *((spif_charptr_t **) SPIFOPT_OPT_VALUE(n)) = tmp;
     } else {
         unsigned short len = argc - i;
 
         /* No equals sign, so use the rest of the command line and break. */
-        tmp = SPIF_CAST_PTR(charptr) MALLOC(sizeof(spif_charptr_t ) * (argc - i + 1));
+        tmp = (spif_charptr_t *) MALLOC(sizeof(spif_charptr_t ) * (argc - i + 1));
 
         for (k = 0; k < len; k++) {
-            tmp[k] = SPIF_CAST(charptr) STRDUP(argv[k + i]);
+            tmp[k] = (spif_charptr_t) STRDUP(argv[k + i]);
             D_OPTIONS(("tmp[%d] == %s\n", k, tmp[k]));
-            argv[k + i] = NULL;
+            if (SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_REMOVE_ARGS)) {
+                argv[k + i] = NULL;
+            }
         }
-        tmp[k] = SPIF_NULL_TYPE(charptr);
-        *(SPIF_CAST_C(spif_charptr_t **) SPIFOPT_OPT_VALUE(n)) = tmp;
+        tmp[k] = (spif_charptr_t) NULL;
+        *((spif_charptr_t **) SPIFOPT_OPT_VALUE(n)) = tmp;
     }
 }
 
@@ -560,7 +558,7 @@ spifopt_parse(int argc, char *argv[])
                 NEXT_LETTER();
             }
         }
-        if (!SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_PREPARSE)) {
+        if (!SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_PREPARSE) && SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_REMOVE_ARGS)) {
             argv[i] = NULL;
         }
 
@@ -582,7 +580,7 @@ spifopt_parse(int argc, char *argv[])
         if (val_ptr) {
             if (val_ptr == SPIF_CHARPTR(argv[i + 1])) {
                 i++;
-                opt += strlen(SPIF_CHARPTR_C(opt));
+                opt += strlen((char *) opt);
             }
         }
 
@@ -600,13 +598,13 @@ spifopt_parse(int argc, char *argv[])
             }
             spif_str_append_from_ptr(warn, SPIFOPT_OPT_LONG(j));
             spif_str_append_from_ptr(warn, SPIF_CHARPTR(" option is deprecated and should not be used.\n"));
-            libast_print_warning(SPIF_CHARPTR_C(SPIF_STR_STR(warn)));
+            libast_print_warning((char *) SPIF_STR_STR(warn));
             spif_str_del(warn);
         }
 
         /* Make sure that options which require a parameter have them. */
         if (SPIFOPT_OPT_NEEDS_VALUE(j)) {
-            if (val_ptr == NULL) {
+            if (!val_ptr) {
                 if (islong) {
                     libast_print_error("long option --%s requires a%s value\n", SPIFOPT_OPT_LONG(j),
                                 (SPIFOPT_OPT_IS_INTEGER(j)
@@ -630,10 +628,10 @@ spifopt_parse(int argc, char *argv[])
                 continue;
             }
             /* Also make sure we know what to do with the value. */
-            if (SPIFOPT_OPT_VALUE(j) == NULL) {
+            if (!SPIFOPT_OPT_VALUE(j)) {
                 NEXT_LOOP();
             }
-        } else if (SPIFOPT_OPT_IS_ABSTRACT(j) && SPIFOPT_OPT_VALUE(j) == NULL) {
+        } else if (SPIFOPT_OPT_IS_ABSTRACT(j) && !SPIFOPT_OPT_VALUE(j)) {
             /* Also make sure that abstract options have a function pointer. */
             NEXT_LOOP();
         }
@@ -663,7 +661,7 @@ spifopt_parse(int argc, char *argv[])
                 ((spifopt_abstract_handler_t) SPIFOPT_OPT_VALUE(j))(val_ptr);
             }
         }
-        if (!SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_PREPARSE)) {
+        if (!SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_PREPARSE) && SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_REMOVE_ARGS)) {
             argv[i] = NULL;
         }
         NEXT_LOOP();
@@ -671,7 +669,7 @@ spifopt_parse(int argc, char *argv[])
 
     if (SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_PREPARSE)) {
         SPIFOPT_FLAGS_CLEAR(SPIFOPT_SETTING_PREPARSE);
-    } else {
+    } else if (SPIFOPT_FLAGS_IS_SET(SPIFOPT_SETTING_REMOVE_ARGS)) {
         for (i = 1, j = 1; i < argc; i++) {
             if (argv[i]) {
                 argv[j] = argv[i];
