@@ -9,8 +9,6 @@
 # undef HAVE_STDARG_H
 #endif
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: snprintf.c,v 1.6 2004/07/23 21:38:39 mej Exp $";
-
 /* 
  * Shamelessly snarfed from Enlightenment...
  * which shamelessly snarfed from sane... 
@@ -239,7 +237,7 @@ fmtstr(char *value, int ljust, int len, int zpad, int precision)
     int padlen, strlen, i, c;   /* amount to pad */
 
     zpad = 0;
-    if (value == 0) {
+    if (!value) {
         value = "<NULL>";
     }
     if (precision > 0) {
@@ -382,7 +380,7 @@ dostr(char *str)
 static void
 dopr_outch(int c)
 {
-    if (end == 0 || output < end) {
+    if (!end || output < end) {
         *output++ = c;
     }
 }
