@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,8 +21,6 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: str.c,v 1.32 2004/12/15 00:00:22 mej Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -75,6 +73,7 @@ static SPIF_CONST_TYPE(strclass) s_class = {
     (spif_func_t) spif_str_rindex,
     (spif_func_t) spif_str_splice,
     (spif_func_t) spif_str_splice_from_ptr,
+    (spif_func_t) spif_str_sprintf,
     (spif_func_t) spif_str_substr,
     (spif_func_t) spif_str_substr_to_ptr,
     (spif_func_t) spif_str_to_float,
@@ -82,11 +81,11 @@ static SPIF_CONST_TYPE(strclass) s_class = {
     (spif_func_t) spif_str_trim,
     (spif_func_t) spif_str_upcase
 };
-SPIF_TYPE(class) SPIF_CLASS_VAR(str) = SPIF_CAST(class) &s_class;
+SPIF_TYPE(class) SPIF_CLASS_VAR(str) = (spif_class_t) &s_class;
 SPIF_TYPE(strclass) SPIF_STRCLASS_VAR(str) = &s_class;
 /* *INDENT-ON* */
 
-const size_t buff_inc = 4096;
+static const size_t buff_inc = 4096;
 
 spif_str_t
 spif_str_new(void)
@@ -96,7 +95,7 @@ spif_str_new(void)
     self = SPIF_ALLOC(str);
     if (!spif_str_init(self)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(str);
+        self = (spif_str_t) NULL;
     }
     return self;
 }
@@ -109,7 +108,7 @@ spif_str_new_from_ptr(spif_charptr_t old)
     self = SPIF_ALLOC(str);
     if (!spif_str_init_from_ptr(self, old)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(str);
+        self = (spif_str_t) NULL;
     }
     return self;
 }
@@ -122,7 +121,7 @@ spif_str_new_from_buff(spif_charptr_t buff, spif_stridx_t size)
     self = SPIF_ALLOC(str);
     if (!spif_str_init_from_buff(self, buff, size)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(str);
+        self = (spif_str_t) NULL;
     }
     return self;
 }
@@ -135,7 +134,7 @@ spif_str_new_from_fp(FILE * fp)
     self = SPIF_ALLOC(str);
     if (!spif_str_init_from_fp(self, fp)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(str);
+        self = (spif_str_t) NULL;
     }
     return self;
 }
@@ -148,7 +147,7 @@ spif_str_new_from_fd(int fd)
     self = SPIF_ALLOC(str);
     if (!spif_str_init_from_fd(self, fd)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(str);
+        self = (spif_str_t) NULL;
     }
     return self;
 }
@@ -161,7 +160,7 @@ spif_str_new_from_num(long num)
     self = SPIF_ALLOC(str);
     if (!spif_str_init_from_num(self, num)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(str);
+        self = (spif_str_t) NULL;
     }
     return self;
 }
@@ -172,7 +171,7 @@ spif_str_init(spif_str_t self)
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_STRCLASS_VAR(str)));
-    self->s = SPIF_NULL_TYPE(charptr);
+    self->s = (spif_charptr_t) NULL;
     self->len = 0;
     self->size = 0;
     return TRUE;
@@ -182,12 +181,12 @@ spif_bool_t
 spif_str_init_from_ptr(spif_str_t self, spif_charptr_t old)
 {
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
-    REQUIRE_RVAL((old != SPIF_NULL_TYPE(charptr)), spif_str_init(self));
+    REQUIRE_RVAL((old != (spif_charptr_t) NULL), spif_str_init(self));
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_STRCLASS_VAR(str)));
-    self->len = strlen(SPIF_CONST_CAST_C(char *) old);
+    self->len = strlen((const char *) old);
     self->size = self->len + 1;
-    self->s = SPIF_CAST(charptr) MALLOC(self->size);
+    self->s = (spif_charptr_t) MALLOC(self->size);
     memcpy(self->s, old, self->size);
     return TRUE;
 }
@@ -199,16 +198,16 @@ spif_str_init_from_buff(spif_str_t self, spif_charptr_t buff, spif_stridx_t size
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_STRCLASS_VAR(str)));
     self->size = size;
-    if (buff != SPIF_NULL_TYPE(charptr)) {
-        self->len = strnlen(SPIF_CONST_CAST_C(char *) buff, size);
+    if (buff != (spif_charptr_t) NULL) {
+        self->len = strnlen((const char *) buff, size);
     } else {
         self->len = 0;
     }
     if (self->size == self->len) {
         self->size++;
     }
-    self->s = SPIF_CAST(charptr) MALLOC(self->size);
-    if (buff != SPIF_NULL_TYPE(charptr)) {
+    self->s = (spif_charptr_t) MALLOC(self->size);
+    if (buff != (spif_charptr_t) NULL) {
         memcpy(self->s, buff, self->len);
     }
     self->s[self->len] = 0;
@@ -221,18 +220,17 @@ spif_str_init_from_fp(spif_str_t self, FILE *fp)
     spif_charptr_t p, end = NULL;
 
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
-    ASSERT_RVAL((fp != SPIF_NULL_TYPE_C(FILE *)), FALSE);
+    ASSERT_RVAL((fp != (FILE *) NULL), FALSE);
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_STRCLASS_VAR(str)));
     self->size = buff_inc;
     self->len = 0;
-    self->s = SPIF_CAST(charptr) MALLOC(self->size);
+    self->s = (spif_charptr_t) MALLOC(self->size);
 
-    for (p = self->s; fgets(SPIF_CAST_C(char *)p, buff_inc, fp); p += buff_inc) {
-        if ((end = SPIF_CAST(charptr)
-             strchr(SPIF_CONST_CAST_C(char *)p, '\n')) == NULL) {
+    for (p = self->s; fgets((char *)p, buff_inc, fp); p += buff_inc) {
+        if (!(end = (spif_charptr_t)strchr((const char *)p, '\n'))) {
             self->size += buff_inc;
-            self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+            self->s = (spif_charptr_t) REALLOC(self->s, self->size);
         } else {
             *end = 0;
             break;
@@ -240,9 +238,9 @@ spif_str_init_from_fp(spif_str_t self, FILE *fp)
     }
     self->len = (spif_stridx_t) ((end)
                           ? (end - self->s)
-                          : ((int) strlen(SPIF_CONST_CAST_C(char *)self->s)));
+                          : ((int) strlen((const char *)self->s)));
     self->size = self->len + 1;
-    self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+    self->s = (spif_charptr_t) REALLOC(self->s, self->size);
     return TRUE;
 }
 
@@ -258,16 +256,16 @@ spif_str_init_from_fd(spif_str_t self, int fd)
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_STRCLASS_VAR(str)));
     self->size = buff_inc;
     self->len = 0;
-    self->s = SPIF_CAST(charptr) MALLOC(self->size);
+    self->s = (spif_charptr_t) MALLOC(self->size);
 
     for (p = self->s; ((n = read(fd, p, buff_inc)) > 0) || (errno == EINTR);) {
         self->size += n;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
         p += n;
     }
     self->len = self->size - buff_inc;
     self->size = self->len + 1;
-    self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+    self->s = (spif_charptr_t) REALLOC(self->s, self->size);
     self->s[self->len] = 0;
     return TRUE;
 }
@@ -281,11 +279,11 @@ spif_str_init_from_num(spif_str_t self, long num)
     /* ***NOT NEEDED*** spif_obj_init(SPIF_OBJ(self)); */
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS(SPIF_STRCLASS_VAR(str)));
 
-    snprintf(SPIF_CHARPTR_C(buff), sizeof(buff), "%ld", num);
-    self->len = strlen(SPIF_CHARPTR_C(buff));
+    snprintf((char *) buff, sizeof(buff), "%ld", num);
+    self->len = strlen((char *) buff);
     self->size = self->len + 1;
-    self->s = SPIF_CAST(charptr) MALLOC(self->size);
-    strcpy(SPIF_CHARPTR_C(self->s), SPIF_CHARPTR_C(buff));
+    self->s = (spif_charptr_t) MALLOC(self->size);
+    strcpy((char *) self->s, (char *) buff);
 
     return TRUE;
 }
@@ -298,7 +296,7 @@ spif_str_done(spif_str_t self)
         FREE(self->s);
         self->len = 0;
         self->size = 0;
-        self->s = SPIF_NULL_TYPE(charptr);
+        self->s = (spif_charptr_t) NULL;
     }
     return TRUE;
 }
@@ -323,9 +321,9 @@ spif_str_show(spif_str_t self, spif_charptr_t name, spif_str_t buff, size_t inde
     }
 
     memset(tmp, ' ', indent);
-    snprintf(SPIF_CHARPTR_C(tmp) + indent, sizeof(tmp) - indent,
+    snprintf((char *) tmp + indent, sizeof(tmp) - indent,
              "(spif_str_t) %s:  %10p { \"",
-             name, SPIF_CAST(ptr) self);
+             name, (spif_ptr_t) self);
     if (SPIF_STR_ISNULL(buff)) {
         buff = spif_str_new_from_ptr(tmp);
     } else {
@@ -334,7 +332,7 @@ spif_str_show(spif_str_t self, spif_charptr_t name, spif_str_t buff, size_t inde
 
     spif_str_append(buff, self);
 
-    snprintf(SPIF_CHARPTR_C(tmp), sizeof(tmp), "\", len %lu, size %lu }\n", (unsigned long) self->len,
+    snprintf((char *) tmp, sizeof(tmp), "\", len %lu, size %lu }\n", (unsigned long) self->len,
              (unsigned long) self->size);
     spif_str_append_from_ptr(buff, tmp);
     return buff;
@@ -351,10 +349,10 @@ spif_str_dup(spif_str_t self)
 {
     spif_str_t tmp;
 
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), SPIF_NULL_TYPE(str));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (spif_str_t) NULL);
     tmp = SPIF_ALLOC(str);
     memcpy(tmp, self, SPIF_SIZEOF_TYPE(str));
-    tmp->s = SPIF_CAST(charptr) STRDUP(SPIF_CONST_CAST_C(char *) SPIF_STR_STR(self));
+    tmp->s = (spif_charptr_t) STRDUP((const char *) SPIF_STR_STR(self));
     tmp->len = self->len;
     tmp->size = self->size;
     return tmp;
@@ -363,7 +361,7 @@ spif_str_dup(spif_str_t self)
 spif_classname_t
 spif_str_type(spif_str_t self)
 {
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), SPIF_CAST(classname) SPIF_NULLSTR_TYPE(classname));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (spif_classname_t) SPIF_NULLSTR_TYPE(classname));
     return SPIF_OBJ_CLASSNAME(self);
 }
 
@@ -374,7 +372,7 @@ spif_str_append(spif_str_t self, spif_str_t other)
     REQUIRE_RVAL(!SPIF_STR_ISNULL(other), FALSE);
     if (other->size && other->len) {
         self->size += other->size - 1;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
         memcpy(self->s + self->len, SPIF_STR_STR(other), other->len + 1);
         self->len += other->len;
     }
@@ -388,7 +386,7 @@ spif_str_append_char(spif_str_t self, spif_char_t c)
     self->len++;
     if (self->size <= self->len) {
         self->size++;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
     }
     self->s[self->len - 1] = c;
     self->s[self->len] = 0;
@@ -401,11 +399,11 @@ spif_str_append_from_ptr(spif_str_t self, spif_charptr_t other)
     spif_stridx_t len;
 
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
-    REQUIRE_RVAL((other != SPIF_NULL_TYPE(charptr)), FALSE);
-    len = strlen(SPIF_CONST_CAST_C(char *) other);
+    REQUIRE_RVAL((other != (spif_charptr_t) NULL), FALSE);
+    len = strlen((const char *) other);
     if (len) {
         self->size += len;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
         memcpy(self->s + self->len, other, len + 1);
         self->len += len;
     }
@@ -418,7 +416,7 @@ spif_str_casecmp(spif_str_t self, spif_str_t other)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strcasecmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(SPIF_STR_STR(other)));
+    c = strcasecmp((char *) SPIF_STR_STR(self), (char *) SPIF_STR_STR(other));
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -428,7 +426,7 @@ spif_str_casecmp_with_ptr(spif_str_t self, spif_charptr_t other)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strcasecmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(other));
+    c = strcasecmp((char *) SPIF_STR_STR(self), (char *) other);
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -447,7 +445,7 @@ spif_str_cmp(spif_str_t self, spif_str_t other)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strcmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(SPIF_STR_STR(other)));
+    c = strcmp((char *) SPIF_STR_STR(self), (char *) SPIF_STR_STR(other));
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -457,7 +455,7 @@ spif_str_cmp_with_ptr(spif_str_t self, spif_charptr_t other)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strcmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(other));
+    c = strcmp((char *) SPIF_STR_STR(self), (char *) other);
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -478,14 +476,14 @@ spif_str_find(spif_str_t self, spif_str_t other)
 {
     char *tmp;
 
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (SPIF_CAST(stridx) -1));
-    REQUIRE_RVAL(!SPIF_STR_ISNULL(other), (SPIF_CAST(stridx) -1));
-    tmp = strstr(SPIF_CONST_CAST_C(char *) SPIF_STR_STR(self),
-                 SPIF_CONST_CAST_C(char *) SPIF_STR_STR(other));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), ((spif_stridx_t) -1));
+    REQUIRE_RVAL(!SPIF_STR_ISNULL(other), ((spif_stridx_t) -1));
+    tmp = strstr((const char *) SPIF_STR_STR(self),
+                 (const char *) SPIF_STR_STR(other));
     if (tmp) {
-        return SPIF_CAST(stridx) (SPIF_CAST(long) tmp - SPIF_CAST(long) (SPIF_STR_STR(self)));
+        return (spif_stridx_t) ((spif_long_t) tmp - (spif_long_t) (SPIF_STR_STR(self)));
     } else {
-        return SPIF_CAST(stridx) (self->len);
+        return (spif_stridx_t) (self->len);
     }
 }
 
@@ -494,14 +492,14 @@ spif_str_find_from_ptr(spif_str_t self, spif_charptr_t other)
 {
     char *tmp;
 
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (SPIF_CAST(stridx) -1));
-    REQUIRE_RVAL((other != SPIF_NULL_TYPE(charptr)), (SPIF_CAST(stridx) -1));
-    tmp = strstr(SPIF_CONST_CAST_C(char *) SPIF_STR_STR(self),
-                 SPIF_CONST_CAST_C(char *) other);
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), ((spif_stridx_t) -1));
+    REQUIRE_RVAL((other != (spif_charptr_t) NULL), ((spif_stridx_t) -1));
+    tmp = strstr((const char *) SPIF_STR_STR(self),
+                 (const char *) other);
     if (tmp) {
-        return SPIF_CAST(stridx) (SPIF_CAST(long) tmp - SPIF_CAST(long) (SPIF_STR_STR(self)));
+        return (spif_stridx_t) ((spif_long_t) tmp - (spif_long_t) (SPIF_STR_STR(self)));
     } else {
-        return SPIF_CAST(stridx) (self->len);
+        return (spif_stridx_t) (self->len);
     }
 }
 
@@ -510,12 +508,12 @@ spif_str_index(spif_str_t self, spif_char_t c)
 {
     char *tmp;
 
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (SPIF_CAST(stridx) -1));
-    tmp = index(SPIF_CONST_CAST_C(char *) SPIF_STR_STR(self), c);
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), ((spif_stridx_t) -1));
+    tmp = index((const char *) SPIF_STR_STR(self), c);
     if (tmp) {
-        return SPIF_CAST(stridx) (SPIF_CAST(long) tmp - SPIF_CAST(long) (SPIF_STR_STR(self)));
+        return (spif_stridx_t) ((spif_long_t) tmp - (spif_long_t) (SPIF_STR_STR(self)));
     } else {
-        return SPIF_CAST(stridx) (self->len);
+        return (spif_stridx_t) (self->len);
     }
 }
 
@@ -525,7 +523,7 @@ spif_str_ncasecmp(spif_str_t self, spif_str_t other, spif_stridx_t cnt)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strncasecmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(SPIF_STR_STR(other)), cnt);
+    c = strncasecmp((char *) SPIF_STR_STR(self), (char *) SPIF_STR_STR(other), cnt);
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -535,7 +533,7 @@ spif_str_ncasecmp_with_ptr(spif_str_t self, spif_charptr_t other, spif_stridx_t
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strncasecmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(other), cnt);
+    c = strncasecmp((char *) SPIF_STR_STR(self), (char *) other, cnt);
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -545,7 +543,7 @@ spif_str_ncmp(spif_str_t self, spif_str_t other, spif_stridx_t cnt)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strncmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(SPIF_STR_STR(other)), cnt);
+    c = strncmp((char *) SPIF_STR_STR(self), (char *) SPIF_STR_STR(other), cnt);
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -555,7 +553,7 @@ spif_str_ncmp_with_ptr(spif_str_t self, spif_charptr_t other, spif_stridx_t cnt)
     int c;
 
     SPIF_OBJ_COMP_CHECK_NULL(self, other);
-    c = strncmp(SPIF_CHARPTR_C(SPIF_STR_STR(self)), SPIF_CHARPTR_C(other), cnt);
+    c = strncmp((char *) SPIF_STR_STR(self), (char *) other, cnt);
     return SPIF_CMP_FROM_INT(c);
 }
 
@@ -566,7 +564,7 @@ spif_str_prepend(spif_str_t self, spif_str_t other)
     REQUIRE_RVAL(!SPIF_STR_ISNULL(other), FALSE);
     if (other->size && other->len) {
         self->size += other->size - 1;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
         memmove(self->s + other->len, self->s, self->len + 1);
         memcpy(self->s, SPIF_STR_STR(other), other->len);
         self->len += other->len;
@@ -581,10 +579,10 @@ spif_str_prepend_char(spif_str_t self, spif_char_t c)
     self->len++;
     if (self->size <= self->len) {
         self->size++;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
     }
     memmove(self->s + 1, self->s, self->len + 1);
-    self->s[0] = SPIF_CAST(uchar) c;
+    self->s[0] = (spif_uchar_t) c;
     return TRUE;
 }
 
@@ -594,11 +592,11 @@ spif_str_prepend_from_ptr(spif_str_t self, spif_charptr_t other)
     spif_stridx_t len;
 
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
-    REQUIRE_RVAL((other != SPIF_NULL_TYPE(charptr)), FALSE);
-    len = strlen(SPIF_CONST_CAST_C(char *) other);
+    REQUIRE_RVAL((other != (spif_charptr_t) NULL), FALSE);
+    len = strlen((const char *) other);
     if (len) {
         self->size += len;
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+        self->s = (spif_charptr_t) REALLOC(self->s, self->size);
         memmove(self->s + len, self->s, self->len + 1);
         memcpy(self->s, other, len);
         self->len += len;
@@ -610,7 +608,7 @@ spif_bool_t
 spif_str_reverse(spif_str_t self)
 {
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
-    return ((strrev(SPIF_CAST_C(char *) self->s)) ? TRUE : FALSE);
+    return ((strrev((char *) self->s)) ? TRUE : FALSE);
 }
 
 spif_stridx_t
@@ -618,12 +616,12 @@ spif_str_rindex(spif_str_t self, spif_char_t c)
 {
     char *tmp;
 
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (SPIF_CAST(stridx) -1));
-    tmp = rindex(SPIF_CONST_CAST_C(char *) SPIF_STR_STR(self), c);
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), ((spif_stridx_t) -1));
+    tmp = rindex((const char *) SPIF_STR_STR(self), c);
     if (tmp) {
-        return SPIF_CAST(stridx) (SPIF_CAST(long) tmp - SPIF_CAST(long) (SPIF_STR_STR(self)));
+        return (spif_stridx_t) ((spif_long_t) tmp - (spif_long_t) (SPIF_STR_STR(self)));
     } else {
-        return SPIF_CAST(stridx) (self->len);
+        return (spif_stridx_t) (self->len);
     }
 }
 
@@ -646,7 +644,7 @@ spif_str_splice(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt, spif_str_
     REQUIRE_RVAL(cnt <= (self->len - idx), FALSE);
 
     newsize = self->len + ((SPIF_STR_ISNULL(other)) ? (0) : (other->len)) - cnt + 1;
-    ptmp = tmp = SPIF_CAST(charptr) MALLOC(newsize);
+    ptmp = tmp = (spif_charptr_t) MALLOC(newsize);
     if (idx > 0) {
         memcpy(tmp, self->s, idx);
         ptmp += idx;
@@ -657,7 +655,7 @@ spif_str_splice(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt, spif_str_
     }
     memcpy(ptmp, self->s + idx + cnt, self->len - idx - cnt + 1);
     if (self->size < newsize) {
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, newsize);
+        self->s = (spif_charptr_t) REALLOC(self->s, newsize);
         self->size = newsize;
     }
     self->len = newsize - 1;
@@ -673,7 +671,7 @@ spif_str_splice_from_ptr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt,
     spif_stridx_t len, newsize;
 
     ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
-    len = (other ? strlen(SPIF_CONST_CAST_C(char *) other) : 0);
+    len = (other ? strlen((const char *) other) : 0);
     if (idx < 0) {
         idx = self->len + idx;
     }
@@ -686,7 +684,7 @@ spif_str_splice_from_ptr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt,
     REQUIRE_RVAL(cnt <= (self->len - idx), FALSE);
 
     newsize = self->len + len - cnt + 1;
-    ptmp = tmp = SPIF_CAST(charptr) MALLOC(newsize);
+    ptmp = tmp = (spif_charptr_t) MALLOC(newsize);
     if (idx > 0) {
         memcpy(tmp, self->s, idx);
         ptmp += idx;
@@ -697,7 +695,7 @@ spif_str_splice_from_ptr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt,
     }
     memcpy(ptmp, self->s + idx + cnt, self->len - idx - cnt + 1);
     if (self->size < newsize) {
-        self->s = SPIF_CAST(charptr) REALLOC(self->s, newsize);
+        self->s = (spif_charptr_t) REALLOC(self->s, newsize);
         self->size = newsize;
     }
     self->len = newsize - 1;
@@ -706,19 +704,59 @@ spif_str_splice_from_ptr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt,
     return TRUE;
 }
 
+spif_bool_t
+spif_str_sprintf(spif_str_t self, spif_charptr_t format, ...)
+{
+    va_list ap;
+
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), FALSE);
+    if (self->s != (spif_charptr_t) NULL) {
+        spif_str_done(self);
+    }
+    if (!format) {
+        return FALSE;
+    } else if (*format == 0) {
+        return TRUE;
+    } else {
+        int c;
+        char buff[2];
+
+        va_start(ap, format);
+        c = vsnprintf(buff, sizeof(buff), format, ap);
+        va_end(ap);
+        if (c <= 0) {
+            return FALSE;
+        } else {
+            self->size = c + 1;
+            self->s = (spif_charptr_t) MALLOC(self->size);
+            va_start(ap, format);
+            c = vsnprintf(self->s, self->size, format, ap);
+            va_end(ap);
+            if (c > -1 && c < self->size) {
+                self->len = c;
+                return TRUE;
+            } else {
+                self->s[0] = 0;
+                return FALSE;
+            }
+        }
+    }
+    ASSERT_NOTREACHED_RVAL(FALSE);
+}
+
 spif_str_t
 spif_str_substr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt)
 {
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), SPIF_NULL_TYPE(str));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (spif_str_t) NULL);
     if (idx < 0) {
         idx = self->len + idx;
     }
-    REQUIRE_RVAL(idx >= 0, SPIF_NULL_TYPE(str));
-    REQUIRE_RVAL(idx < self->len, SPIF_NULL_TYPE(str));
+    REQUIRE_RVAL(idx >= 0, (spif_str_t) NULL);
+    REQUIRE_RVAL(idx < self->len, (spif_str_t) NULL);
     if (cnt <= 0) {
         cnt = self->len - idx + cnt;
     }
-    REQUIRE_RVAL(cnt >= 0, SPIF_NULL_TYPE(str));
+    REQUIRE_RVAL(cnt >= 0, (spif_str_t) NULL);
     UPPER_BOUND(cnt, self->len - idx);
     return spif_str_new_from_buff(SPIF_STR_STR(self) + idx, cnt);
 }
@@ -728,19 +766,19 @@ spif_str_substr_to_ptr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt)
 {
     spif_charptr_t newstr;
 
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), SPIF_NULL_TYPE(charptr));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (spif_charptr_t) NULL);
     if (idx < 0) {
         idx = self->len + idx;
     }
-    REQUIRE_RVAL(idx >= 0, SPIF_NULL_TYPE(charptr));
-    REQUIRE_RVAL(idx < self->len, SPIF_NULL_TYPE(charptr));
+    REQUIRE_RVAL(idx >= 0, (spif_charptr_t) NULL);
+    REQUIRE_RVAL(idx < self->len, (spif_charptr_t) NULL);
     if (cnt <= 0) {
         cnt = self->len - idx + cnt;
     }
-    REQUIRE_RVAL(cnt >= 0, SPIF_NULL_TYPE(charptr));
+    REQUIRE_RVAL(cnt >= 0, (spif_charptr_t) NULL);
     UPPER_BOUND(cnt, self->len - idx);
 
-    newstr = SPIF_CAST(charptr) MALLOC(cnt + 1);
+    newstr = (spif_charptr_t) MALLOC(cnt + 1);
     memcpy(newstr, SPIF_STR_STR(self) + idx, cnt);
     newstr[cnt] = 0;
     return newstr;
@@ -749,15 +787,15 @@ spif_str_substr_to_ptr(spif_str_t self, spif_stridx_t idx, spif_stridx_t cnt)
 double
 spif_str_to_float(spif_str_t self)
 {
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), SPIF_CAST_C(double) NAN);
-    return (double) (strtod(SPIF_CONST_CAST_C(char *)SPIF_STR_STR(self), (char **) NULL));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (double) NAN);
+    return (double) (strtod((const char *)SPIF_STR_STR(self), (char **) NULL));
 }
 
 size_t
 spif_str_to_num(spif_str_t self, int base)
 {
-    ASSERT_RVAL(!SPIF_STR_ISNULL(self), (SPIF_CAST_C(size_t) -1));
-    return (size_t) (strtoul(SPIF_CONST_CAST_C(char *) SPIF_STR_STR(self), (char **) NULL, base));
+    ASSERT_RVAL(!SPIF_STR_ISNULL(self), ((size_t) -1));
+    return (size_t) (strtoul((const char *) SPIF_STR_STR(self), (char **) NULL, base));
 }
 
 spif_bool_t
@@ -777,7 +815,7 @@ spif_str_trim(spif_str_t self)
     self->len = (spif_stridx_t) (end - start);
     self->size = self->len + 1;
     memmove(self->s, start, self->size);
-    self->s = SPIF_CAST(charptr) REALLOC(self->s, self->size);
+    self->s = (spif_charptr_t) REALLOC(self->s, self->size);
     return TRUE;
 }
 
