@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,8 +21,6 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-static const char __attribute__((unused)) cvs_ident[] = "$Id: url.c,v 1.20 2004/07/23 21:38:39 mej Exp $";
-
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
@@ -54,7 +52,7 @@ spif_url_new(void)
     self = SPIF_ALLOC(url);
     if (!spif_url_init(self)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(url);
+        self = (spif_url_t) NULL;
     }
     return self;
 }
@@ -67,7 +65,7 @@ spif_url_new_from_str(spif_str_t other)
     self = SPIF_ALLOC(url);
     if (!spif_url_init_from_str(self, other)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(url);
+        self = (spif_url_t) NULL;
     }
     return self;
 }
@@ -80,7 +78,7 @@ spif_url_new_from_ptr(spif_charptr_t other)
     self = SPIF_ALLOC(url);
     if (!spif_url_init_from_ptr(self, other)) {
         SPIF_DEALLOC(self);
-        self = SPIF_NULL_TYPE(url);
+        self = (spif_url_t) NULL;
     }
     return self;
 }
@@ -93,13 +91,13 @@ spif_url_init(spif_url_t self)
         return FALSE;
     }
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS_VAR(url));
-    self->proto = SPIF_NULL_TYPE(str);
-    self->user = SPIF_NULL_TYPE(str);
-    self->passwd = SPIF_NULL_TYPE(str);
-    self->host = SPIF_NULL_TYPE(str);
-    self->port = SPIF_NULL_TYPE(str);
-    self->path = SPIF_NULL_TYPE(str);
-    self->query = SPIF_NULL_TYPE(str);
+    self->proto = (spif_str_t) NULL;
+    self->user = (spif_str_t) NULL;
+    self->passwd = (spif_str_t) NULL;
+    self->host = (spif_str_t) NULL;
+    self->port = (spif_str_t) NULL;
+    self->path = (spif_str_t) NULL;
+    self->query = (spif_str_t) NULL;
     return TRUE;
 }
 
@@ -111,13 +109,13 @@ spif_url_init_from_str(spif_url_t self, spif_str_t other)
         return FALSE;
     }
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS_VAR(url));
-    self->proto = SPIF_NULL_TYPE(str);
-    self->user = SPIF_NULL_TYPE(str);
-    self->passwd = SPIF_NULL_TYPE(str);
-    self->host = SPIF_NULL_TYPE(str);
-    self->port = SPIF_NULL_TYPE(str);
-    self->path = SPIF_NULL_TYPE(str);
-    self->query = SPIF_NULL_TYPE(str);
+    self->proto = (spif_str_t) NULL;
+    self->user = (spif_str_t) NULL;
+    self->passwd = (spif_str_t) NULL;
+    self->host = (spif_str_t) NULL;
+    self->port = (spif_str_t) NULL;
+    self->path = (spif_str_t) NULL;
+    self->query = (spif_str_t) NULL;
     spif_url_parse(self);
     return TRUE;
 }
@@ -130,13 +128,13 @@ spif_url_init_from_ptr(spif_url_t self, spif_charptr_t other)
         return FALSE;
     }
     spif_obj_set_class(SPIF_OBJ(self), SPIF_CLASS_VAR(url));
-    self->proto = SPIF_NULL_TYPE(str);
-    self->user = SPIF_NULL_TYPE(str);
-    self->passwd = SPIF_NULL_TYPE(str);
-    self->host = SPIF_NULL_TYPE(str);
-    self->port = SPIF_NULL_TYPE(str);
-    self->path = SPIF_NULL_TYPE(str);
-    self->query = SPIF_NULL_TYPE(str);
+    self->proto = (spif_str_t) NULL;
+    self->user = (spif_str_t) NULL;
+    self->passwd = (spif_str_t) NULL;
+    self->host = (spif_str_t) NULL;
+    self->port = (spif_str_t) NULL;
+    self->path = (spif_str_t) NULL;
+    self->query = (spif_str_t) NULL;
     spif_url_parse(self);
     return TRUE;
 }
@@ -147,31 +145,31 @@ spif_url_done(spif_url_t self)
     ASSERT_RVAL(!SPIF_URL_ISNULL(self), FALSE);
     if (!SPIF_STR_ISNULL(self->proto)) {
         spif_str_del(self->proto);
-        self->proto = SPIF_NULL_TYPE(str);
+        self->proto = (spif_str_t) NULL;
     }
     if (!SPIF_STR_ISNULL(self->user)) {
         spif_str_del(self->user);
-        self->user = SPIF_NULL_TYPE(str);
+        self->user = (spif_str_t) NULL;
     }
     if (!SPIF_STR_ISNULL(self->passwd)) {
         spif_str_del(self->passwd);
-        self->passwd = SPIF_NULL_TYPE(str);
+        self->passwd = (spif_str_t) NULL;
     }
     if (!SPIF_STR_ISNULL(self->host)) {
         spif_str_del(self->host);
-        self->host = SPIF_NULL_TYPE(str);
+        self->host = (spif_str_t) NULL;
     }
     if (!SPIF_STR_ISNULL(self->port)) {
         spif_str_del(self->port);
-        self->port = SPIF_NULL_TYPE(str);
+        self->port = (spif_str_t) NULL;
     }
     if (!SPIF_STR_ISNULL(self->path)) {
         spif_str_del(self->path);
-        self->path = SPIF_NULL_TYPE(str);
+        self->path = (spif_str_t) NULL;
     }
     if (!SPIF_STR_ISNULL(self->query)) {
         spif_str_del(self->query);
-        self->query = SPIF_NULL_TYPE(str);
+        self->query = (spif_str_t) NULL;
     }
     spif_str_done(SPIF_STR(self));
     return TRUE;
@@ -197,9 +195,9 @@ spif_url_show(spif_url_t self, spif_charptr_t name, spif_str_t buff, size_t inde
     }
 
     memset(tmp, ' ', indent);
-    snprintf(SPIF_CHARPTR_C(tmp) + indent, sizeof(tmp) - indent,
+    snprintf((char *) tmp + indent, sizeof(tmp) - indent,
              "(spif_url_t) %s:  %10p {\n",
-             name, SPIF_CAST(ptr) self);
+             name, (spif_ptr_t) self);
     if (SPIF_STR_ISNULL(buff)) {
         buff = spif_str_new_from_ptr(tmp);
     } else {
@@ -214,7 +212,7 @@ spif_url_show(spif_url_t self, spif_charptr_t name, spif_str_t buff, size_t inde
     buff = spif_str_show(self->path, SPIF_CHARPTR("path"), buff, indent + 2);
     buff = spif_str_show(self->query, SPIF_CHARPTR("query"), buff, indent + 2);
 
-    snprintf(SPIF_CHARPTR_C(tmp) + indent, sizeof(tmp) - indent, "}\n");
+    snprintf((char *) tmp + indent, sizeof(tmp) - indent, "}\n");
     spif_str_append_from_ptr(buff, tmp);
     return buff;
 }
@@ -231,7 +229,7 @@ spif_url_dup(spif_url_t self)
 {
     spif_url_t tmp;
 
-    ASSERT_RVAL(!SPIF_URL_ISNULL(self), SPIF_NULL_TYPE(url));
+    ASSERT_RVAL(!SPIF_URL_ISNULL(self), (spif_url_t) NULL);
     tmp = spif_url_new_from_str(SPIF_STR(self));
     return tmp;
 }
@@ -239,7 +237,7 @@ spif_url_dup(spif_url_t self)
 spif_classname_t
 spif_url_type(spif_url_t self)
 {
-    ASSERT_RVAL(!SPIF_URL_ISNULL(self), SPIF_NULL_TYPE(classname));
+    ASSERT_RVAL(!SPIF_URL_ISNULL(self), (spif_classname_t) NULL);
     return SPIF_OBJ_CLASSNAME(self);
 }
 
@@ -261,8 +259,8 @@ spif_url_parse(spif_url_t self)
     pstr = s;
 
     /* Check for "proto:" at the beginning. */
-    pend = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(s), ':'));
-    if (pend != NULL) {
+    pend = SPIF_CHARPTR(strchr((char *) s, ':'));
+    if (pend) {
         for (; pstr < pend; pstr++) {
             if (!isalnum(*pstr)) {
                 break;
@@ -283,17 +281,17 @@ spif_url_parse(spif_url_t self)
     }
 
     /* Knock out the path and query if they're there. */
-    pend = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(pstr), '/'));
-    if (pend != NULL) {
-        spif_charptr_t tmp = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(pend), '?'));
+    pend = SPIF_CHARPTR(strchr((char *) pstr, '/'));
+    if (pend) {
+        spif_charptr_t tmp = SPIF_CHARPTR(strchr((char *) pend, '?'));
 
-        if (tmp != NULL) {
+        if (tmp) {
             self->query = spif_str_new_from_ptr(tmp + 1);
             self->path = spif_str_new_from_buff(pend, tmp - pend);
         } else {
           self->path = spif_str_new_from_ptr(pend);
         }
-    } else if ((pend = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(pstr), '?'))) != NULL) {
+    } else if ((pend = SPIF_CHARPTR(strchr((char *)pstr, '?')))) {
         self->query = spif_str_new_from_ptr(pend + 1);
     } else {
         for (pend = pstr; *pend; pend++);
@@ -301,11 +299,11 @@ spif_url_parse(spif_url_t self)
     /* At this point, pend *must* point to the end of the user/pass/host/port part. */
 
     /* Check for an @ sign, which would mean we have auth info. */
-    ptmp = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(pstr), '@'));
-    if ((ptmp != NULL) && (ptmp < pend)) {
-        spif_charptr_t tmp = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(pstr), ':'));
+    ptmp = SPIF_CHARPTR(strchr((char *) pstr, '@'));
+    if ((ptmp) && (ptmp < pend)) {
+        spif_charptr_t tmp = SPIF_CHARPTR(strchr((char *) pstr, ':'));
 
-        if ((tmp != NULL) && (tmp < ptmp)) {
+        if ((tmp) && (tmp < ptmp)) {
             /* Both username and password. */
             self->user = spif_str_new_from_buff(pstr, tmp - pstr);
             self->passwd = spif_str_new_from_buff((tmp + 1), ptmp - tmp - 1);
@@ -316,8 +314,8 @@ spif_url_parse(spif_url_t self)
     }
 
     /* All that remains now between pstr and pend is host and maybe port. */
-    ptmp = SPIF_CHARPTR(strchr(SPIF_CHARPTR_C(pstr), ':'));
-    if ((ptmp != NULL) && (ptmp < pend)) {
+    ptmp = SPIF_CHARPTR(strchr((char *) pstr, ':'));
+    if ((ptmp) && (ptmp < pend)) {
         self->host = spif_str_new_from_buff(pstr, ptmp - pstr);
         self->port = spif_str_new_from_buff((ptmp + 1), pend - ptmp - 1);
     } else if (pstr != pend) {
@@ -329,22 +327,22 @@ spif_url_parse(spif_url_t self)
         spif_protoinfo_t proto;
         spif_servinfo_t serv;
 
-        proto = getprotobyname(SPIF_CHARPTR_C(SPIF_STR_STR(self->proto)));
-        if (proto == NULL) {
+        proto = getprotobyname((char *) SPIF_STR_STR(self->proto));
+        if (!proto) {
             /* If it's not a protocol, it's probably a service. */
-            serv = getservbyname(SPIF_CHARPTR_C(SPIF_STR_STR(self->proto)), "tcp");
-            if (serv == NULL) {
-                serv = getservbyname(SPIF_CHARPTR_C(SPIF_STR_STR(self->proto)), "udp");
+            serv = getservbyname((char *) SPIF_STR_STR(self->proto), "tcp");
+            if (!serv) {
+                serv = getservbyname((char *) SPIF_STR_STR(self->proto), "udp");
             }
-            if (serv != NULL) {
+            if (serv) {
                 proto = getprotobyname(serv->s_proto);
                 REQUIRE_RVAL(proto != NULL, FALSE);
             }
         }
-        if (proto != NULL) {
+        if (proto) {
             spif_char_t buff[32];
 
-            snprintf(SPIF_CHARPTR_C(buff), sizeof(buff), "%d", ntohs(serv->s_port));
+            snprintf((char *) buff, sizeof(buff), "%d", ntohs(serv->s_port));
             self->port = spif_str_new_from_ptr(buff);
         }
     }
