@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1997-2004, Michael Jennings
+ * Copyright (C) 1997-2013, Michael Jennings <mej@eterm.org>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -21,8 +21,6 @@
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
 
-static const char cvs_ident[] = "$Id: perf.c,v 1.13 2004/10/26 18:01:54 mej Exp $";
-
 #if defined(HAVE_CONFIG_H) && (HAVE_CONFIG_H != 0)
 # include <config.h>
 #endif
@@ -178,7 +176,7 @@ perf_hashes(void)
             spif_uint8_t buff[KEYLEN];
 
             for (key_length = 0; key_length < KEYLEN; key_length++) {
-                buff[key_length] = SPIF_CAST(uint8) (rand() & 0xff);
+                buff[key_length] = (spif_uint8_t) (rand() & 0xff);
             }
             PERF_BEGIN("hash speed for 32-byte key");
             if (hash_func == spifhash_jenkins32) {
@@ -195,7 +193,7 @@ perf_hashes(void)
             spif_uint8_t buff[KEYLEN];
 
             for (key_length = 0; key_length < KEYLEN; key_length++) {
-                buff[key_length] = SPIF_CAST(uint8) (rand() & 0xff);
+                buff[key_length] = (spif_uint8_t) (rand() & 0xff);
             }
             PERF_BEGIN("hash speed for 512-byte key");
             if (hash_func == spifhash_jenkins32) {
@@ -212,7 +210,7 @@ perf_hashes(void)
             spif_uint8_t buff[KEYLEN];
 
             for (key_length = 0; key_length < KEYLEN; key_length++) {
-                buff[key_length] = SPIF_CAST(uint8) (rand() & 0xff);
+                buff[key_length] = (spif_uint8_t) (rand() & 0xff);
             }
             PERF_BEGIN("hash speed for 8192-byte key");
             if (hash_func == spifhash_jenkins32) {
@@ -238,20 +236,20 @@ perf_hashes(void)
 #define KEYCNT      64
         do {
             spif_uint32_t buckets[SPIFHASH_SIZE(HASH_BITS)];
-            spif_uint32_t c_total, c_min = SPIF_CAST(uint32) (-1), c_max = 0;
+            spif_uint32_t c_total, c_min = (spif_uint32_t) (-1), c_max = 0;
             double c_prob, avg_size;
 
             MEMSET(buckets, 0, sizeof(buckets));
             printf("Profiling %lu-bucket distribution for %lu-bit hashes of %lu %lu-byte random words...",
-                   SPIF_CAST_C(unsigned long) SPIFHASH_SIZE(HASH_BITS),
-                   SPIF_CAST_C(unsigned long) HASH_BITS,
-                   SPIF_CAST_C(unsigned long) KEYCNT,
-                   SPIF_CAST_C(unsigned long) KEYLEN);
+                   (unsigned long) SPIFHASH_SIZE(HASH_BITS),
+                   (unsigned long) HASH_BITS,
+                   (unsigned long) KEYCNT,
+                   (unsigned long) KEYLEN);
             for (j = 0; j < KEYCNT; j++) {
                 spif_uint8_t buff[KEYLEN];
 
                 for (key_length = 0; key_length < KEYLEN; key_length++) {
-                    buff[key_length] = SPIF_CAST(uint8) (rand() % 26 + 'a');
+                    buff[key_length] = (spif_uint8_t) (rand() % 26 + 'a');
                 }
                 if (hash_func == spifhash_jenkins32) {
                     hash = hash_func(buff, KEYLEN / 4, hash);
@@ -275,11 +273,11 @@ perf_hashes(void)
                 }
             }
             avg_size /= SPIFHASH_SIZE(HASH_BITS);
-            c_prob = ((SPIF_CAST_C(double) c_total) / KEYCNT * 100.0);
+            c_prob = (((double) c_total) / KEYCNT * 100.0);
             printf("%lu collisions (%lu/%lu/%.5le), %.2lf%% probability.\n",
-                   SPIF_CAST_C(unsigned long) c_total,
-                   SPIF_CAST_C(unsigned long) c_min,
-                   SPIF_CAST_C(unsigned long) c_max,
+                   (unsigned long) c_total,
+                   (unsigned long) c_min,
+                   (unsigned long) c_max,
                    avg_size, c_prob);
         } while (0);
 #undef KEYCNT
@@ -287,20 +285,20 @@ perf_hashes(void)
 #define KEYCNT      2048
         do {
             spif_uint32_t buckets[SPIFHASH_SIZE(HASH_BITS)];
-            spif_uint32_t c_total, c_min = SPIF_CAST(uint32) (-1), c_max = 0;
+            spif_uint32_t c_total, c_min = (spif_uint32_t) (-1), c_max = 0;
             double c_prob, avg_size;
 
             MEMSET(buckets, 0, sizeof(buckets));
             printf("Profiling %lu-bucket distribution for %lu-bit hashes of %lu %lu-byte random words...",
-                   SPIF_CAST_C(unsigned long) SPIFHASH_SIZE(HASH_BITS),
-                   SPIF_CAST_C(unsigned long) HASH_BITS,
-                   SPIF_CAST_C(unsigned long) KEYCNT,
-                   SPIF_CAST_C(unsigned long) KEYLEN);
+                   (unsigned long) SPIFHASH_SIZE(HASH_BITS),
+                   (unsigned long) HASH_BITS,
+                   (unsigned long) KEYCNT,
+                   (unsigned long) KEYLEN);
             for (j = 0; j < KEYCNT; j++) {
                 spif_uint8_t buff[KEYLEN];
 
                 for (key_length = 0; key_length < KEYLEN; key_length++) {
-                    buff[key_length] = SPIF_CAST(uint8) (rand() % 26 + 'a');
+                    buff[key_length] = (spif_uint8_t) (rand() % 26 + 'a');
                 }
                 if (hash_func == spifhash_jenkins32) {
                     hash = hash_func(buff, KEYLEN / 4, hash);
@@ -324,11 +322,11 @@ perf_hashes(void)
                 }
             }
             avg_size /= SPIFHASH_SIZE(HASH_BITS);
-            c_prob = ((SPIF_CAST_C(double) c_total) / KEYCNT * 100.0);
+            c_prob = (((double) c_total) / KEYCNT * 100.0);
             printf("%lu collisions (%lu/%lu/%.5le), %.2lf%% probability.\n",
-                   SPIF_CAST_C(unsigned long) c_total,
-                   SPIF_CAST_C(unsigned long) c_min,
-                   SPIF_CAST_C(unsigned long) c_max,
+                   (unsigned long) c_total,
+                   (unsigned long) c_min,
+                   (unsigned long) c_max,
                    avg_size, c_prob);
         } while (0);
 #undef KEYCNT
@@ -336,20 +334,20 @@ perf_hashes(void)
 #define KEYCNT      32768
         do {
             spif_uint32_t buckets[SPIFHASH_SIZE(HASH_BITS)];
-            spif_uint32_t c_total, c_min = SPIF_CAST(uint32) (-1), c_max = 0;
+            spif_uint32_t c_total, c_min = (spif_uint32_t) (-1), c_max = 0;
             double c_prob, avg_size;
 
             MEMSET(buckets, 0, sizeof(buckets));
             printf("Profiling %lu-bucket distribution for %lu-bit hashes of %lu %lu-byte random words...",
-                   SPIF_CAST_C(unsigned long) SPIFHASH_SIZE(HASH_BITS),
-                   SPIF_CAST_C(unsigned long) HASH_BITS,
-                   SPIF_CAST_C(unsigned long) KEYCNT,
-                   SPIF_CAST_C(unsigned long) KEYLEN);
+                   (unsigned long) SPIFHASH_SIZE(HASH_BITS),
+                   (unsigned long) HASH_BITS,
+                   (unsigned long) KEYCNT,
+                   (unsigned long) KEYLEN);
             for (j = 0; j < KEYCNT; j++) {
                 spif_uint8_t buff[KEYLEN];
 
                 for (key_length = 0; key_length < KEYLEN; key_length++) {
-                    buff[key_length] = SPIF_CAST(uint8) (rand() % 26 + 'a');
+                    buff[key_length] = (spif_uint8_t) (rand() % 26 + 'a');
                 }
                 if (hash_func == spifhash_jenkins32) {
                     hash = hash_func(buff, KEYLEN / 4, hash);
@@ -373,11 +371,11 @@ perf_hashes(void)
                 }
             }
             avg_size /= SPIFHASH_SIZE(HASH_BITS);
-            c_prob = ((SPIF_CAST_C(double) c_total) / KEYCNT * 100.0);
+            c_prob = (((double) c_total) / KEYCNT * 100.0);
             printf("%lu collisions (%lu/%lu/%.5le), %.2lf%% probability.\n",
-                   SPIF_CAST_C(unsigned long) c_total,
-                   SPIF_CAST_C(unsigned long) c_min,
-                   SPIF_CAST_C(unsigned long) c_max,
+                   (unsigned long) c_total,
+                   (unsigned long) c_min,
+                   (unsigned long) c_max,
                    avg_size, c_prob);
         } while (0);
 #undef KEYCNT
@@ -412,7 +410,7 @@ perf_options(void)
     char *display = NULL, *name = NULL, *theme = NULL, **exec = NULL, **foo = NULL;
     long color = 0;
     spif_uint32_t options = 0;
-    static void handle_theme(char *val_ptr) {theme = STRDUP(val_ptr);}
+    void handle_theme(char *val_ptr) {theme = STRDUP(val_ptr);}
     char *argv2[] = { "test", "-rt", "mytheme", "--name", "This is a name", "--exec=ssh foo@bar.com", "--scrollbar",
                       "--buttonbar", "no", "--login=0", "-mvd", "foo:0", "--color", "4", "--foo", "blah", "-d", "eatme", NULL };
     int argc2 = 18;
@@ -529,14 +527,14 @@ perf_str(void)
     PERF_TEST(spif_str_rindex(teststr, '#'););
     PERF_END();
 
-    test2str = spif_str_new_from_ptr(SPIF_CAST(charptr) "ring");
+    test2str = spif_str_new_from_ptr((spif_charptr_t) "ring");
     PERF_BEGIN("spif_str_find() function");
     PERF_TEST(spif_str_find(teststr, test2str););
     PERF_END();
     spif_str_del(test2str);
 
     PERF_BEGIN("spif_str_find_from_ptr() function");
-    PERF_TEST(spif_str_find_from_ptr(teststr, SPIF_CAST(charptr) "in"););
+    PERF_TEST(spif_str_find_from_ptr(teststr, (spif_charptr_t) "in"););
     PERF_END();
 
     spif_str_del(teststr);
@@ -555,13 +553,13 @@ perf_str(void)
     PERF_END();
     spif_str_del(teststr);
 
-    teststr = spif_str_new_from_ptr(SPIF_CAST(charptr) "11001001");
+    teststr = spif_str_new_from_ptr((spif_charptr_t) "11001001");
     PERF_BEGIN("spif_str_to_num() function");
     PERF_TEST(spif_str_to_num(teststr, 10););
     PERF_END();
     spif_str_del(teststr);
 
-    teststr = spif_str_new_from_ptr(SPIF_CAST(charptr) "3.1415");
+    teststr = spif_str_new_from_ptr((spif_charptr_t) "3.1415");
     PERF_BEGIN("spif_str_to_float() function");
     PERF_TEST(spif_str_to_float(teststr););
     PERF_END();
@@ -589,13 +587,13 @@ perf_str(void)
 
     teststr = spif_str_new_from_ptr("abcdefg");
     PERF_BEGIN("spif_str_clear() function");
-    PERF_TEST(spif_str_clear(teststr, SPIF_CAST_C(char) (rand() & 0xff)););
+    PERF_TEST(spif_str_clear(teststr, (char) (rand() & 0xff)););
     PERF_END();
     spif_str_del(teststr);
 
     PERF_BEGIN("spif_str_trim() function");
     PERF_TEST(
-              teststr = spif_str_new_from_ptr(SPIF_CAST(charptr) "  \n \r\f       \t    testing 1 2 3    \v\r \n");
+              teststr = spif_str_new_from_ptr((spif_charptr_t) "  \n \r\f       \t    testing 1 2 3    \v\r \n");
               spif_str_trim(teststr);
               spif_str_del(teststr);
               );
@@ -654,7 +652,6 @@ perf_tok(void)
     testtok = spif_tok_new_from_ptr(tmp2);
     teststr = spif_str_new_from_ptr(":");
     spif_tok_set_sep(testtok, teststr);
-    spif_str_del(teststr);
     PERF_TEST(spif_tok_eval(testtok););
     spif_tok_del(testtok);
 
@@ -681,7 +678,7 @@ perf_url(void)
     spif_charptr_t tmp3 = "/path/to/some/file.jpg";
     spif_charptr_t tmp4 = "pop3://dummy:moo@pop.nowhere.com:110";
 
-    PERF_SET_REPS(100);
+    PERF_SET_REPS(10);
 
     PERF_BEGIN("spif_url_new_from_ptr() function");
     PERF_TEST(testurl = spif_url_new_from_ptr(tmp1); spif_url_del(testurl););
@@ -719,18 +716,18 @@ perf_list(void)
         }
 
         PERF_BEGIN("SPIF_LIST_APPEND() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         PERF_TEST(SPIF_LIST_APPEND(testlist, spif_str_new_from_ptr(buff)););
         PERF_END();
 
         PERF_BEGIN("SPIF_LIST_PREPEND() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         PERF_TEST(SPIF_LIST_PREPEND(testlist, spif_str_new_from_ptr(buff)););
         PERF_END();
 
         s = spif_str_new();
         PERF_BEGIN("SPIF_LIST_CONTAINS() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         spif_str_init_from_ptr(s, buff);
         PERF_TEST(SPIF_LIST_CONTAINS(testlist, s));
         spif_str_done(s);
@@ -743,13 +740,13 @@ perf_list(void)
 
         len = SPIF_LIST_COUNT(testlist);
         PERF_BEGIN("SPIF_LIST_GET() macro");
-        idx = SPIF_CAST_C(size_t) (rand() % len);
+        idx = (size_t) (rand() % len);
         PERF_TEST(SPIF_LIST_GET(testlist, idx));
         PERF_END();
 
         s = spif_str_new();
         PERF_BEGIN("SPIF_LIST_INDEX() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         spif_str_init_from_ptr(s, buff);
         PERF_TEST(SPIF_LIST_INDEX(testlist, s));
         spif_str_done(s);
@@ -757,29 +754,29 @@ perf_list(void)
         spif_str_del(s);
 
         PERF_BEGIN("SPIF_LIST_INSERT() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         PERF_TEST(SPIF_LIST_INSERT(testlist, spif_str_new_from_ptr(buff)););
         PERF_END();
 
         len = SPIF_LIST_COUNT(testlist);
         PERF_BEGIN("SPIF_LIST_INSERT_AT() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         idx = rand() % len;
         PERF_TEST(SPIF_LIST_INSERT_AT(testlist, spif_str_new_from_ptr(buff), idx););
         PERF_END();
 
         s = spif_str_new();
         PERF_BEGIN("SPIF_LIST_REMOVE() macro");
-        buff[0] = SPIF_CAST_C(char) (rand() & 0xff);
+        buff[0] = (char) (rand() & 0xff);
         spif_str_init_from_ptr(s, buff);
-        PERF_TEST(s2 = SPIF_CAST(str) SPIF_LIST_REMOVE(testlist, s); if (!SPIF_STR_ISNULL(s2)) {spif_str_del(s2);});
+        PERF_TEST(s2 = (spif_str_t) SPIF_LIST_REMOVE(testlist, s); if (!SPIF_STR_ISNULL(s2)) {spif_str_del(s2);});
         spif_str_done(s);
         PERF_END();
         spif_str_del(s);
 
         PERF_BEGIN("SPIF_LIST_REMOVE_AT() macro");
         idx = rand() % SPIF_LIST_COUNT(testlist);
-        PERF_TEST(s2 = SPIF_CAST(str) SPIF_LIST_REMOVE_AT(testlist, idx); if (!SPIF_STR_ISNULL(s2)) {spif_str_del(s2);});
+        PERF_TEST(s2 = (spif_str_t) SPIF_LIST_REMOVE_AT(testlist, idx); if (!SPIF_STR_ISNULL(s2)) {spif_str_del(s2);});
         PERF_END();
 
         /*SPIF_SHOW(testlist, stderr);*/
